@@ -34,8 +34,8 @@ from pkscreener.classes import VERSION
 configManager = ConfigManager.tools()
 MENU_SEPARATOR = ""
 LINE_SEPARATOR = "\n"
-MAX_SUPPORTED_MENU_OPTION = 33
-MAX_MENU_OPTION = 41
+MAX_SUPPORTED_MENU_OPTION = 38
+MAX_MENU_OPTION = 42
 
 level0MenuDict = {
     "X": "Scanners",
@@ -55,51 +55,108 @@ level0MenuDict = {
     "H": "Help / About Developer",
     "Z": "Exit (Ctrl + C)",
 }
+level1_index_options_sectoral= {
+    "1": "Nifty 50 (^NSEI)                                  ",
+    "2": "BSE Sensex (^BSESN)",
+    "3": "Nifty Auto (^CNXAUTO)                             ",
+    "4": "Nifty Bank (^NSEBANK)",
+    "5": "Nifty Consumption (^CNXCONSUM)                    ",
+    "6": "Nifty Energy (^CNXENERGY)",
+    "7": "Nifty Financial Services (NIFTY_FIN_SERVICE.NS)   ",
+    "8": "Nifty Financial Services 25/50 (^CNXFIN)",
+    "9": "Nifty FMCG (^CNXFMCG)                             ",
+    "10": "Nifty Healthcare (NIFTY_HEALTHCARE.NS)",
+    "11": "Nifty IT (^CNXIT)                                ",
+    "12": "Nifty Infra (^CNXINFRA)",
+    "13": "Nifty Media (^CNXMEDIA)                           ",
+    "14": "Nifty Metal (^CNXMETAL)",
+    "15": "Nifty Pharma (^CNXPHARMA)                        ",
+    "16": "Nifty Private Bank (NIFTY_PVT_BANK.NS)",
+    "17": "Nifty PSU Bank (^CNXPSUBANK)                     ",
+    "18": "Nifty Realty (^CNXREALTY)",
+    "19": "Nifty Service Sector (^CNXSERVICE)               ",
+    "20": "Nifty MidCap 50 (^NSEMDCP50)",
+    "21": "Nifty MidCap 100 (NIFTY_MIDCAP_100.NS)           ",
+    "22": "Nifty MidCap 100 (NIFTY_LARGEMID_250.NS)",
+    "23": "Nifty 100 ESG (NIFTY100_ESG.NS)                  ",
+    "24": "Nifty Consumer Durables (NIFTY_CONSR_DURBL.NS)",
+    "25": "Nifty Oil and Gas (NIFTY_OIL_AND_GAS.NS)         ",
+    "26": "Nifty MidSmall Healthcare (NIFTY_MIDSML_HLTH.NS)",
+    "27": "All of the above",
+}
 level1_P_MenuDict = {
     "1": "Predefined Piped Scanners",
     "2": "Define my custom Piped Scanner",
     "3": "Run Piped Scans Saved So Far",
     "M": "Back to the Top/Main menu",
 }
-PREDEFINED_SCAN_MENU_KEYS = ["1","2","3","4","5","6","7","8","9","10","11","12","13","14","15"]
+PREDEFINED_SCAN_MENU_KEYS = ["1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20", "21", "22", "23", "24", "25"]
 PREDEFINED_SCAN_MENU_TEXTS = [
-    "Volume Scanners | High Momentum | Breaking Out Now | ATR Cross     ",
-    "Volume Scanners | High Momentum | ATR Cross",
-    "Volume Scanners | High Momentum                                    ",
-    "Volume Scanners | ATR Cross",
-    "Volume Scanners | High Bid/Ask Build Up                            ",
-    "High Momentum | ATR Cross",
-    "High Momentum | ATR Trailing Stop                                  ",
-    "ATR Cross | ATR Trailing Stop",
-    "TTM Sqeeze Buy | Intraday RSI b/w 0 to 54                          ",
-    "Volume Scanners | High Momentum | Breaking Out Now | ATR Cross | Intraday RSI b/w 0 to 54",
-    "Volume Scanners | ATR Cross | Intraday RSI b/w 0 to 54             ",
-    "VCP (Mark Minervini) | Chart Patterns | MA Support",
-    "VCP | Chart Patterns | MA Support                                  ",
-    "Already Breaking out | VCP (Minervini) | Chart Patterns | MA Support",
-    "ATR Trailing Stops | VCP (Minervini)                               ",
+    "Volume Scanners | High Momentum | Breaking Out Now | ATR Cross     ",  # 1
+    "Volume Scanners | High Momentum | ATR Cross",                          # 2
+    "Volume Scanners | High Momentum                                    ",  # 3
+    "Volume Scanners | ATR Cross",                                          # 4
+    "Volume Scanners | High Bid/Ask Build Up                            ",  # 5
+    "Volume Scanners | ATR Cross | ATR Trailing Stops",                     # 6
+    "Volume Scanners | ATR Trailing Stops                               ",  # 7
+    "High Momentum | ATR Cross",                                            # 8
+    "High Momentum | ATR Trailing Stop                                  ",  # 9
+    "ATR Cross | ATR Trailing Stop",                                        # 10
+    "TTM Sqeeze Buy | RSI b/w 0 to 54                                   ",  # 11
+    "Volume Scanners | High Momentum | Breaking Out Now | ATR Cross | RSI b/w 0 to 54", # 12
+    "Volume Scanners | ATR Cross | RSI b/w 0 to 54                      ",  # 13
+    "VCP (Mark Minervini) | Chart Patterns | MA Support",                   # 14
+    "VCP | Chart Patterns | MA Support                                  ",  # 15
+    "Already Breaking out | VCP (Minervini) | Chart Patterns | MA Support", # 16
+    "ATR Trailing Stops | VCP (Minervini)                               ",  # 17
+    "VCP | ATR Trailing Stops",                                             # 18
+    "Nifty 50,Nifty Bank | VCP | ATR Trailing Stops                     ",  # 19
+    "Volume Scanners | High Momentum | Breaking Out Now | ATR Cross | VCP | ATR Trailing Stops", # 20
+    "BullCross-MA | Fair Value Buy Opportunities                        ",  # 21
+    "VCP | Chart Patterns | MA Support | Bullish AVWAP",                    # 22
+    "VCP (Mark Minervini) | Chart Patterns | MA Support | Bullish AVWAP ",  # 23
+    "BullCross-VWAP | Volume Scanners",                                     # 24
+    "BullCross-VWAP | ATR Cross | ATR Trailing Stop                     ",  # 25
 ]
 level2_P_MenuDict = {}
 for key in PREDEFINED_SCAN_MENU_KEYS:
     level2_P_MenuDict[key] = PREDEFINED_SCAN_MENU_TEXTS[int(key)-1]
 level2_P_MenuDict["M"] = "Back to the Top/Main menu"
 PREDEFINED_SCAN_MENU_VALUES =[
-    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:>|X:0:23:>|X:0:27:'",
-    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:>|X:0:27:'",
-    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:'",
-    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:27:'",
-    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:29:'",
-    "--systemlaunched -a y -e -o 'X:12:31:>|X:0:27:'",
-    "--systemlaunched -a y -e -o 'X:12:31:>|X:0:30:1:'",
-    "--systemlaunched -a y -e -o 'X:12:27:>|X:0:30:1:'",
-    "--systemlaunched -a y -e -o 'X:12:7:6:1:>|X:0:5:0:54:i 1m'",
-    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:>|X:0:23:>|X:0:27:>|X:0:5:0:54:i 1m'",
-    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:27:>|X:0:5:0:54:i 1m'",
-    "--systemlaunched -a y -e -o 'X:12:7:8:>|X:12:7:9:1:1:'",
-    "--systemlaunched -a y -e -o 'X:12:7:4:>|X:12:7:9:1:1:'",
-    "--systemlaunched -a y -e -o 'X:12:2:>|X:12:7:8:>|X:12:7:9:1:1:'",
-    "--systemlaunched -a y -e -o 'X:12:30:1:>|X:12:7:8:'"
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:>|X:0:23:>|X:0:27:'", # 1
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:>|X:0:27:'",          # 2
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:'",                   # 3
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:27:'",                   # 4
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:29:'",                   # 5
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:27:>|X:12:30:1:'",       # 6
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:12:30:1:'",                # 7
+    "--systemlaunched -a y -e -o 'X:12:31:>|X:0:27:'",                      # 8
+    "--systemlaunched -a y -e -o 'X:12:31:>|X:0:30:1:'",                    # 9
+    "--systemlaunched -a y -e -o 'X:12:27:>|X:0:30:1:'",                    # 10
+    "--systemlaunched -a y -e -o 'X:12:7:6:1:>|X:0:5:0:54:'",               # 11
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:>|X:0:23:>|X:0:27:>|X:0:5:0:54:'", # 12
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:27:>|X:0:5:0:54:'",      # 13
+    "--systemlaunched -a y -e -o 'X:12:7:8:>|X:12:7:9:1:1:'",               # 14
+    "--systemlaunched -a y -e -o 'X:12:7:4:>|X:12:7:9:1:1:'",               # 15
+    "--systemlaunched -a y -e -o 'X:12:2:>|X:12:7:8:>|X:12:7:9:1:1:'",      # 16
+    "--systemlaunched -a y -e -o 'X:12:30:1:>|X:12:7:8:'",                  # 17
+    "--systemlaunched -a y -e -o 'X:12:7:4:>|X:12:30:1:'",                  # 18
+    "--systemlaunched -a y -e -o 'X:0:0:^NSEI,^NSEBANK:>|X:12:7:4:>|X:12:30:1:'", # 19
+    "--systemlaunched -a y -e -o 'X:12:9:2.5:>|X:0:31:>|X:0:23:>|X:0:27:>|X:12:7:4:>|X:12:30:1:'",  # 20
+    "--systemlaunched -a y -e -o 'X:12:7:9:5:>|X:12:21:8:'",                # 21
+    "--systemlaunched -a y -e -o 'X:12:7:4:>|X:12:7:9:1:1:>|X:12:34:'",     # 22
+    "--systemlaunched -a y -e -o 'X:12:7:8:>|X:12:7:9:1:1:>|X:12:34:'",     # 23
+    "--systemlaunched -a y -e -o 'X:12:7:9:7:>|X:0:9:2.5:'",                # 24
+    "--systemlaunched -a y -e -o 'X:12:7:9:7:>|X:0:31:>|X:0:30:1:'",        # 25
 ]
+PREDEFINED_PIPED_MENU_OPTIONS = []
+for option in PREDEFINED_SCAN_MENU_VALUES:
+    argOptions = option.split("-o ")[-1]
+    analysisOptions = argOptions.split("|")
+    analysisOptions[-1] = analysisOptions[-1].replace("X:","C:")
+    argOptions = "|".join(analysisOptions)
+    PREDEFINED_PIPED_MENU_OPTIONS.append(argOptions.replace("'","").replace("\"",""))
+
 PIPED_SCANNERS = {}
 for key in PREDEFINED_SCAN_MENU_KEYS:
     PIPED_SCANNERS[key] = PREDEFINED_SCAN_MENU_VALUES[int(key)-1]
@@ -107,6 +164,7 @@ for key in PREDEFINED_SCAN_MENU_KEYS:
 level1_T_MenuDict = {
     "L": "Long Term",
     "S": "Short Term (Intraday)",
+    "B": "Quick Backtest for N-days/candles ago",
     "M": "Back to the Top/Main menu",
 }
 # Valid periods: 1d,5d,1mo,3mo,6mo,1y,2y,5y,10y,ytd,max
@@ -136,7 +194,8 @@ level1_X_MenuDict = {
     "W": "Screen stocks from my own Watchlist",
     "N": "Nifty Prediction using Artifical Intelligence (Use for Gap-Up/Gap-Down/BTST/STBT)",
     "E": "Live Index Scan : 5 EMA for Intraday",
-    "0": "Screen stocks/index by the stock/index names (NSE Stock Code or index, e.g. ^NSEI, ^NSEBANK, ^BSESN)",
+    "S": "Sectoral Indices",
+    "0": "Screen stocks by the stock names",
     "1": "Nifty 50          ",
     "2": "Nifty Next 50     ",
     "3": "Nifty 100         ",
@@ -188,10 +247,13 @@ level2_X_MenuDict = {
     "29": "Intraday Bid/Ask Build-up      ",
     "30": "ATR Trailing Stops(Swing Paper Trading)",
     "31": "High Momentum(RSI,MFI,CCI)     ",
-    # "32": "High Momentum(14)",
-    # "28": "Extremely bullish daily close      ",
-    # "29": "Rising RSI                      ",
-    # "30": "RSI entering bullish territory",
+    "32": "Intraday Breakout/Breakdown setup",
+    "33": "Potential Profitable setups    ",
+    "34": "Bullish Anchored-VWAP",
+    "35": "Perfect Short Sells (Futures)  ",
+    "36": "Probable Short Sells (Futures)",
+    "37": "Short Sell Candidates (Volume SMA)",
+    "38": "Intraday Short Sell (PSAR / Volume SMA)",
     "42": "Show Last Screened Results",
     "M": "Back to the Top/Main menu",
     "Z": "Exit (Ctrl + C)",
@@ -229,6 +291,7 @@ level4_X_ChartPattern_MASignalMenuDict = {
     "4": "BearCross MA",
     "5": "BullCross MA",
     "6": "MA-Resist",
+    "7": "BullCross VWAP",
     "0": "Cancel",
 }
 
@@ -243,7 +306,8 @@ level4_X_ChartPattern_BBands_SQZ_MenuDict = {
 level4_X_ChartPattern_Confluence_MenuDict = {
     "1": "Confluence up / GoldenCrossOver / DMA50 / DMA200",
     "2": "Confluence Down / DeadCrossOver",
-    "3": "Any/All",
+    "3": "Any/All (Confluence up/down/Crossovers)",
+    "4": "8,21,55-EMA / 200-SMA Super-Confluence",
     "0": "Cancel",
 }
 
@@ -377,12 +441,13 @@ class menus:
     def allMenus(topLevel="X",index=12):
         menuOptions = [topLevel]
         indexOptions =[index]
-        scanOptions = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,23,24,25,27,28,30,31]
+        scanOptions = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,23,24,25,27,28,30,31,32]
         scanSubOptions = {
                             6:[1,2,3,4,5,6,{7:[1,2]},8,9,10],
-                            7:[1,2,{3:[1,2]},4,5,{6:[1,3]},7],
+                            7:[1,2,{3:[1,2]},4,5,{6:[1,3]},7,8,{9:[1,2,3,4,5,6]}],
+                            21:[3,5,6,7,8,9],
                             30:[1,2],
-                            # 21:[3,5,6,7,8,9]
+                            32:[1,2],
                          }
         runOptions = []
         for menuOption in menuOptions:
@@ -534,6 +599,13 @@ class menus:
                         renderStyle=renderStyle,
                         parent=selectedMenu,
                     )
+                elif selectedMenu.menuKey == "S":
+                    return self.renderLevel2_Sectoral_Menus(
+                        skip=skip,
+                        asList=asList,
+                        renderStyle=renderStyle,
+                        parent=selectedMenu,
+                    )
                 else:
                     # next levelsub-menu of the selected sub-menu
                     return self.renderLevel2_X_Menus(
@@ -573,7 +645,7 @@ class menus:
                         renderStyle=renderStyle,
                         parent=selectedMenu,
                     )
-                elif selectedMenu.menuKey in ["30"]:
+                elif selectedMenu.menuKey in ["30","32"]:
                     return self.renderLevel4_X_Lorenzian_Menus(
                         skip=skip,
                         asList=asList,
@@ -740,13 +812,13 @@ class menus:
         defaultKey = 'L' if configManager.period == '1y' else 'S'
         menuText = self.fromDictionary(
             level1_T_MenuDict,
-            renderExceptionKeys=["M"],
+            renderExceptionKeys=["M","B"],
             renderStyle=renderStyle
             if renderStyle is not None
             else MenuRenderStyle.STANDALONE,
             skip=skip,
             parent=parent,
-        ).render(asList=asList,coloredValues=[defaultKey] if not asList else [])
+        ).render(asList=asList,coloredValues=[defaultKey,"B"] if not asList else [])
         if asList:
             return menuText
         else:
@@ -839,13 +911,49 @@ class menus:
                     "" + colorText.END
                 )
             return menuText
-        
+
+    def renderLevel2_Sectoral_Menus(
+        self, skip=[], asList=False, renderStyle=None, parent=None
+    ):
+        indexKeys = level1_index_options_sectoral.keys()
+        menuText = self.fromDictionary(
+            level1_index_options_sectoral,
+            renderExceptionKeys=[str(len(indexKeys))],
+            renderStyle=renderStyle
+            if renderStyle is not None
+            else MenuRenderStyle.TWO_PER_ROW,
+            skip=skip,
+            parent=parent,
+        ).render(asList=asList,coloredValues=[str(len(indexKeys))] if not asList else [])
+        if asList:
+            return menuText
+        else:
+            if OutputControls().enableMultipleLineOutput:
+                OutputControls().printOutput(
+                    colorText.BOLD
+                    + colorText.WARN
+                    + "[+] Select a sectoral index:"
+                    + colorText.END
+                )
+                OutputControls().printOutput(
+                    colorText.BOLD
+                    + menuText
+                    + """
+
+    Enter your choice > (default is """
+                    + colorText.WARN
+                    + (self.find(str(len(indexKeys))) or menu().create('?','?')).keyTextLabel().strip()
+                    + ")  "
+                    "" + colorText.END
+                )
+            return menuText
+
     def renderLevel1_X_Menus(
         self, skip=[], asList=False, renderStyle=None, parent=None
     ):
         menuText = self.fromDictionary(
             level1_X_MenuDict,
-            renderExceptionKeys=["W", "0", "M", "15"],
+            renderExceptionKeys=["W", "0", "M", "S", "15"],
             renderStyle=renderStyle
             if renderStyle is not None
             else MenuRenderStyle.THREE_PER_ROW,
@@ -1229,7 +1337,7 @@ class menus:
             else MenuRenderStyle.STANDALONE,
             skip=skip,
             parent=parent,
-        ).render(asList=asList,coloredValues=["1"] if not asList else [])
+        ).render(asList=asList,coloredValues=["4"] if not asList else [])
         if asList:
             return menuText
         else:
@@ -1247,7 +1355,7 @@ class menus:
 
     Enter your choice > (default is """
                     + colorText.WARN
-                    + (self.find("1") or menu().create('?','?')).keyTextLabel().strip() + ")"
+                    + (self.find("4") or menu().create('?','?')).keyTextLabel().strip() + ")"
                     + colorText.END
                 )
             return menuText
