@@ -38,8 +38,13 @@ from PKDevTools.classes import Archiver
 from PKDevTools.classes.Singleton import SingletonType, SingletonMixin
 from PKDevTools.classes.PKPickler import PKPicklerDB
 from PKDevTools.classes.PKDateUtilities import PKDateUtilities
+from PKDevTools.classes.MarketHours import MarketHours
 from PKDevTools.classes.log import default_logger
 from PKDevTools.classes.OutputControls import OutputControls
+from PKDevTools.classes.SuppressOutput import SuppressOutput
+from PKDevTools.classes.MarketHours import MarketHours
+
+configManager = tools()
 
 STD_ENCODING=sys.stdout.encoding if sys.stdout is not None else 'utf-8'
 
@@ -90,10 +95,13 @@ class PKMarketOpenCloseAnalyser:
         # stockListFromMorningTrade,morningIntraday_df = PKMarketOpenCloseAnalyser.simulateMorningTrade(updatedCandleData)
         # latest_daily_df = PKMarketOpenCloseAnalyser.runScanForStocksFromMorningTrade(stockListFromMorningTrade,allDailyCandles)
         try:
-            PKMarketOpenCloseAnalyser.diffMorningCandleDataWithLatestDailyCandleData(screen_df,save_df, updatedCandleData, allDailyCandles,runOptionName=runOptionName,filteredListOfStocks=filteredListOfStocks)
+            shouldSuppress = not OutputControls().enableMultipleLineOutput
+            with SuppressOutput(suppress_stderr=shouldSuppress, suppress_stdout=shouldSuppress):
+                save_df, screen_df = PKMarketOpenCloseAnalyser.diffMorningCandleDataWithLatestDailyCandleData(screen_df,save_df, updatedCandleData, allDailyCandles,runOptionName=runOptionName,filteredListOfStocks=filteredListOfStocks)
         except:
             pass
         Utility.tools.saveStockData(allDailyCandles,PKMarketOpenCloseAnalyser.configManager,1,False,False, True)
+        return save_df, screen_df
 
     def ensureIntradayStockDataExists():
         # Ensure that the intraday_stock_data_<date>.pkl file exists
@@ -101,7 +109,7 @@ class PKMarketOpenCloseAnalyser:
         copyFilePath = os.path.join(Archiver.get_user_outputs_dir(), f"copy_{cache_file}")
         srcFilePath = os.path.join(Archiver.get_user_outputs_dir(), cache_file)
         srcFileSize = os.stat(srcFilePath).st_size if os.path.exists(srcFilePath) else 0
-        if exists and srcFileSize < 1024*1024*50:
+        if exists and srcFileSize < 1024*1024*40:
              # File less than 30MB ? Must have been corrupted
             try:
                 os.remove(srcFilePath)
@@ -115,10 +123,15 @@ class PKMarketOpenCloseAnalyser:
             PKMarketOpenCloseAnalyser.configManager.duration = "1m"
             OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()} !")
             OutputControls().printOutput(f"[+] {colorText.GREEN}Trying to download {cache_file}{colorText.END}. Please wait ...")
-            Utility.tools.loadStockData({},PKMarketOpenCloseAnalyser.configManager,False,'Y',False,False,None,isIntraday=True)
+            if os.path.exists(copyFilePath):
+                copyFileSize = os.stat(copyFilePath).st_size if os.path.exists(copyFilePath) else 0
+                if copyFileSize >= 1024*1024*40:
+                    shutil.copy(copyFilePath,srcFilePath) # copy is the saved source of truth
+                    return True, cache_file
+            Utility.tools.loadStockData({},PKMarketOpenCloseAnalyser.configManager,False,'Y',False,False,[],isIntraday=True)
             exists, cache_file = Utility.tools.afterMarketStockDataExists(intraday=True)
             if not exists:
-                OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()} !")
+                OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()}/ !")
                 OutputControls().printOutput(f"[+] Please run {colorText.FAIL}pkscreener{colorText.END}{colorText.GREEN} -a Y -e -d -i 1m{colorText.END} and then run this menu option again.")
                 input("Press any key to continue...")
             PKMarketOpenCloseAnalyser.configManager.period = savedPeriod
@@ -138,7 +151,7 @@ class PKMarketOpenCloseAnalyser:
         copyFilePath = os.path.join(Archiver.get_user_outputs_dir(), f"copy_{cache_file}")
         srcFilePath = os.path.join(Archiver.get_user_outputs_dir(), cache_file)
         srcFileSize = os.stat(srcFilePath).st_size if os.path.exists(srcFilePath) else 0
-        if exists and srcFileSize < 1024*1024*50:
+        if exists and srcFileSize < 1024*1024*40:
              # File less than 30MB ? Must have been corrupted
             try:
                 os.remove(srcFilePath)
@@ -150,10 +163,15 @@ class PKMarketOpenCloseAnalyser:
         # We should download a fresh copy anyways because we may have altered the existing copy in
         # the previous run. -- !!!! Not required if we saved at the end of last operation !!!!
             OutputControls().printOutput(f"[+] {colorText.GREEN}Trying to download {cache_file}{colorText.END}. Please wait ...")
-            Utility.tools.loadStockData({},PKMarketOpenCloseAnalyser.configManager,False,'Y',False,False,None,isIntraday=False,forceRedownload=True)
+            if os.path.exists(copyFilePath):
+                copyFileSize = os.stat(copyFilePath).st_size if os.path.exists(copyFilePath) else 0
+                if copyFileSize >= 1024*1024*40:
+                    shutil.copy(copyFilePath,srcFilePath) # copy is the saved source of truth
+                    return True, cache_file
+            Utility.tools.loadStockData({},PKMarketOpenCloseAnalyser.configManager,False,'Y',False,False,[],isIntraday=False,forceRedownload=True)
             exists, cache_file = Utility.tools.afterMarketStockDataExists(intraday=False)
             if not exists:
-                OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()} !")
+                OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()}/ !")
                 OutputControls().printOutput(f"[+] Please run {colorText.FAIL}pkscreener{colorText.END}{colorText.GREEN} -a Y -e -d{colorText.END} and then run this menu option again.")
                 input("Press any key to continue...")
         try:
@@ -220,9 +238,9 @@ class PKMarketOpenCloseAnalyser:
                                 index=allDailyIntradayCandles[stock]["index"])
                 df = df.head(numOfCandles)
                 try:
-                    df = df[df.index <=  pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} 09:{15+candle1MinuteNumberSinceMarketStarted}:00+05:30').to_datetime64()]
+                    df = df[df.index <=  pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+candle1MinuteNumberSinceMarketStarted}:00+05:30').to_datetime64()]
                 except:
-                    df = df[df.index <=  pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} 09:{15+candle1MinuteNumberSinceMarketStarted}:00+05:30', utc=True)]
+                    df = df[df.index <=  pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+candle1MinuteNumberSinceMarketStarted}:00+05:30', utc=True)]
                     pass
                 if df is not None and len(df) > 0:
                     combinedCandle = {"Open":PKMarketOpenCloseAnalyser.getMorningOpen(df), "High":max(df["High"]), 
@@ -302,6 +320,17 @@ class PKMarketOpenCloseAnalyser:
         save_df.drop(f"index", axis=1, inplace=True, errors="ignore")
         screen_df.drop(f"index", axis=1, inplace=True, errors="ignore")
         stocks = save_df["Stock"]
+        filterStocks = []
+        for stock in stocks:
+            if stock in filteredListOfStocks:
+                filterStocks.append(stock)
+        stocks = filterStocks
+        save_df = save_df[save_df['Stock'].isin(filteredListOfStocks)]
+        df_screenResults = None
+        for stk in filteredListOfStocks:
+            df_screenResults_filter = screen_df[screen_df['Stock'].astype(str).str.contains(f"NSE%3A{stk}") == True]
+            df_screenResults = pd.concat([df_screenResults, df_screenResults_filter], axis=0)
+        screen_df = df_screenResults
         eodLTPs = []
         dayHighLTPs = []
         morningTimestamps = []
@@ -313,6 +342,8 @@ class PKMarketOpenCloseAnalyser:
         dayHighDiffs = []
         sqrOffDiffs = []
         index = 0
+        ts = None
+        row = None
         scrStats = ScreeningStatistics(PKMarketOpenCloseAnalyser.configManager, default_logger())
         for stock in stocks:
             try:
@@ -337,7 +368,28 @@ class PKMarketOpenCloseAnalyser:
                                            afterTimestamp=updatedCandleData[stock]["index"][-1],
                                            nthCrossover=1,
                                            upDirection=True)
+                # saveDictionary = {}
+                # screeningDictionary = {}
+                # nextSellMinute = 1
+                # foundNextSellCandle = False
+                # index = None
+                # while not foundNextSellCandle:
+                #     try:
+                #         # Let's only consider those candles that are right after the alert issue-time in the mornings
+                #         index = pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+PKMarketOpenCloseAnalyser.configManager.morninganalysiscandlenumber+nextSellMinute}:00+05:30').to_datetime64()
+                #         df = df[df.index <=  index]
+                #     except:
+                #         index = pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+PKMarketOpenCloseAnalyser.configManager.morninganalysiscandlenumber+nextSellMinute}:00+05:30', utc=True)
+                #         df = df[df.index <=  index]
+                #         pass
+                #     foundNextSellCandle = scrStats.findATRTrailingStops(df=df,sensitivity=configManager.atrTrailingStopSensitivity, atr_period=configManager.atrTrailingStopPeriod,ema_period=configManager.atrTrailingStopEMAPeriod,buySellAll=2,saveDict=saveDictionary,screenDict=screeningDictionary)
+                #     nextSellMinute += 1
+                # if foundNextSellCandle:
+                #     ts = df.tail(len(df)-index +1).head(1).index[-1]
+                #     row = df[df.index == ts]
                 highTS, highRow = scrStats.findIntradayHighCrossover(df=df)
+                # buySell_df = scrStats.computeBuySellSignals(updatedCandleData[stock]["data"])
+                # OutputControls().printOutput(buySell_df)
                 dayHighLTP = dayHighLTP if pd.notna(dayHighLTP) else highRow["High"][-1]
                 sellTimestamps.append(ts.strftime("%H:%M"))
                 dayHighTimestamps.append(highTS.strftime("%H:%M"))
@@ -357,35 +409,36 @@ class PKMarketOpenCloseAnalyser:
                 continue
         diffColumns = ["LTP@Alert", "AlertTime", "SqrOff", "SqrOffLTP", "SqrOffDiff","DayHighTime","DayHigh","DayHighDiff", "EoDLTP", "EoDDiff"]
         diffValues = [morningAlertLTPs, morningTimestamps, sellTimestamps, sellLTPs, sqrOffDiffs,dayHighTimestamps,dayHighLTPs, dayHighDiffs,eodLTPs, eodDiffs]
-        for col in diffColumns:
-            save_df[col] = diffValues[diffColumns.index(col)]
-            screen_df.loc[:, col] = save_df.loc[:, col].apply(
-                lambda x: x if col in ["LTP@Alert","AlertTime", "SqrOff", "SqrOffLTP", "EoDLTP","DayHigh","DayHighTime"] else ((colorText.GREEN if x >= 0 else colorText.FAIL) + str(x) + colorText.END)
+        for column in diffColumns:
+            columnName = column
+            save_df[columnName] = diffValues[diffColumns.index(columnName)]
+            screen_df.loc[:, columnName] = save_df.loc[:, columnName].apply(
+                lambda x: x if columnName in ["LTP@Alert","AlertTime", "SqrOff", "SqrOffLTP", "EoDLTP","DayHigh","DayHighTime"] else ((colorText.GREEN if x >= 0 else colorText.FAIL) + str(x) + colorText.END)
             )
 
         columns = save_df.columns
         lastIndex = len(save_df)
         for col in columns:
-            if col in ["Stock", "Pattern", "LTP", "SqrOffLTP","SqrOffDiff","DayHigh","DayHighDiff", "EoDLTP", "EoDDiff", "%Chng"]:
+            if col in ["Stock", "LTP@Alert", "Pattern", "LTP", "SqrOffLTP","SqrOffDiff","DayHigh","DayHighDiff", "EoDLTP", "EoDDiff", "%Chng"]:
                 if col == "Stock":
-                    save_df.loc[lastIndex,col] = "PORTFOLIO"
+                    save_df.loc[lastIndex,col] = "BASKET"
                 elif col == "Pattern":
                     save_df.loc[lastIndex,col] = runOptionName if runOptionName is not None else ""
-                elif col in ["LTP", "SqrOffLTP","SqrOffDiff", "EoDLTP", "EoDDiff","DayHigh","DayHighDiff"]:
+                elif col in ["LTP", "LTP@Alert", "SqrOffLTP","SqrOffDiff", "EoDLTP", "EoDDiff","DayHigh","DayHighDiff"]:
                     save_df.loc[lastIndex,col] = round(sum(save_df[col].dropna(inplace=False).astype(float)),2)
                 elif col == "%Chng":
-                    ltpSum = sum(save_df["LTP"].dropna(inplace=False).astype(float))
+                    ltpSum = sum(save_df["LTP@Alert"].dropna(inplace=False).astype(float))
                     change_pct = sum(save_df["EoDDiff"].dropna(inplace=False).astype(float))*100/ltpSum
-                    save_df.loc[lastIndex,col] = f"{round(2*change_pct,2)}%" # when summing above for LTP, the total of LTP gets summed up too. Hence *2 here.
+                    save_df.loc[lastIndex,col] = f"{round(change_pct,2)}%"
             else:
                 save_df.loc[lastIndex,col] = ""
             screen_df.loc[lastIndex,col] = save_df.loc[lastIndex,col]
         eodDiff = save_df.loc[lastIndex,"EoDDiff"]
         sqrOffDiff = save_df.loc[lastIndex,"SqrOffDiff"]
         dayHighDiff = save_df.loc[lastIndex,"DayHighDiff"]
-        save_df.loc[lastIndex,"EoDDiff"] = str(eodDiff) + f'({round(100*2*eodDiff/ltpSum,2)}%)'
-        save_df.loc[lastIndex,"SqrOffDiff"] = str(sqrOffDiff) + f'({round(100*2*sqrOffDiff/ltpSum,2)}%)'
-        save_df.loc[lastIndex,"DayHighDiff"] = str(dayHighDiff) + f'({round(100*2*dayHighDiff/ltpSum,2)}%)'
+        save_df.loc[lastIndex,"EoDDiff"] = str(eodDiff) + f'({round(100*eodDiff/ltpSum,2)}%)'
+        save_df.loc[lastIndex,"SqrOffDiff"] = str(sqrOffDiff) + f'({round(100*sqrOffDiff/ltpSum,2)}%)'
+        save_df.loc[lastIndex,"DayHighDiff"] = str(dayHighDiff) + f'({round(100*dayHighDiff/ltpSum,2)}%)'
         screen_df.loc[lastIndex,"EoDDiff"] = (colorText.GREEN if eodDiff >= 0 else colorText.FAIL) + save_df.loc[lastIndex,"EoDDiff"] + colorText.END
         screen_df.loc[lastIndex,"SqrOffDiff"] = (colorText.GREEN if sqrOffDiff >= 0 else colorText.FAIL) + save_df.loc[lastIndex,"SqrOffDiff"] + colorText.END
         screen_df.loc[lastIndex,"DayHighDiff"] = (colorText.GREEN if dayHighDiff >= 0 else colorText.FAIL) + save_df.loc[lastIndex,"DayHighDiff"] + colorText.END
@@ -394,7 +447,11 @@ class PKMarketOpenCloseAnalyser:
         PKMarketOpenCloseAnalyser.allIntradayCandles = None
         screen_df.replace(np.nan, "", regex=True)
         save_df.replace(np.nan, "", regex=True)
-        if 'index' in save_df.columns:
-            save_df.drop('index', axis=1, inplace=True, errors="ignore")
-        if 'index' in screen_df.columns:
-            screen_df.drop('index', axis=1, inplace=True, errors="ignore")
+        # Drop the unnecessary columns for this scanner type to make way for other columns to be fitted nicely on screen
+        columnsToBeDropped = ["Breakout(22Prds)","MA-Signal","Trend(22Prds)","index","EoDLTP"]
+        for col in columnsToBeDropped:
+            if col in save_df.columns:
+                save_df.drop(col, axis=1, inplace=True, errors="ignore")
+            if col in screen_df.columns:
+                screen_df.drop(col, axis=1, inplace=True, errors="ignore")
+        return save_df, screen_df
