@@ -65,8 +65,8 @@ class PKScanRunner:
                 "Consol.",
                 "Breakout",
                 "LTP",
-                "52Wk H",
-                "52Wk L",
+                "52Wk-H",
+                "52Wk-L",
                 "%Chng",
                 "Volume",
                 "MA-Signal",
@@ -83,8 +83,8 @@ class PKScanRunner:
                 "Consol.",
                 "Breakout",
                 "LTP",
-                "52Wk H",
-                "52Wk L",
+                "52Wk-H",
+                "52Wk-L",
                 "%Chng",
                 "Volume",
                 "MA-Signal",
