@@ -98,11 +98,12 @@ class StockScreener:
                 hostRef.processingCounter.value += 1
 
             volumeRatio, period = self.determineBasicConfigs(stock, newlyListedOnly, volumeRatio, logLevel, hostRef, configManager, screener, userArgsLog)
-            if userArgsLog:
-                hostRef.default_logger.info(f"For stock:{stock}, stock exists in objectDictionary:{hostRef.objectDictionaryPrimary.get(stock)}, cacheEnabled:{configManager.cacheEnabled}, isTradingTime:{self.isTradingTime}, downloadOnly:{downloadOnly}")
+            # if userArgsLog:
+            #     hostRef.default_logger.info(f"For stock:{stock}, stock exists in objectDictionary:{hostRef.objectDictionaryPrimary.get(stock)}, cacheEnabled:{configManager.cacheEnabled}, isTradingTime:{self.isTradingTime}, downloadOnly:{downloadOnly}")
             data = None
+            intraday_data = None
             data = self.getRelevantDataForStock(totalSymbols, shouldCache, stock, downloadOnly, printCounter, backtestDuration, hostRef,hostRef.objectDictionaryPrimary, configManager, fetcher, period,None, testData,exchangeName)
-            if not configManager.isIntradayConfig() and configManager.calculatersiintraday:
+            if str(executeOption) in ["32","38"] or (not configManager.isIntradayConfig() and configManager.calculatersiintraday):
                 # Daily data is already available in "data" above.
                 # We need the intraday data for 1-d RSI values when config is not for intraday
                 intraday_data = self.getRelevantDataForStock(totalSymbols, shouldCache, stock, downloadOnly, printCounter, backtestDuration, hostRef, hostRef.objectDictionarySecondary, configManager, fetcher, "1d","1m", testData,exchangeName)
@@ -231,7 +232,7 @@ class StockScreener:
                 raise StockDataEmptyException("Empty processedData")
             suppressError = (logLevel==logging.NOTSET)
             suppressOut = (not (printCounter or testbuild))
-            with SuppressOutput(suppress_stderr=False, suppress_stdout=False):
+            with SuppressOutput(suppress_stderr=suppressError, suppress_stdout=suppressOut):
                 self.updateStock(stock, screeningDictionary, saveDictionary, executeOption, exchangeName)
                 
                 self.performBasicLTPChecks(executeOption, screeningDictionary, saveDictionary, fullData, configManager, screener, exchangeName)
@@ -266,7 +267,7 @@ class StockScreener:
                 hasBbandsSqz = False
                 hasMASignalFilter = False
 
-                isValidityCheckMet = self.performValidityCheckForExecuteOptions(executeOption,screener,fullData,screeningDictionary,saveDictionary,processedData,configManager,maLength)
+                isValidityCheckMet = self.performValidityCheckForExecuteOptions(executeOption,screener,fullData,screeningDictionary,saveDictionary,processedData,configManager,maLength,intraday_data)
                 if not isValidityCheckMet:
                     return returnLegibleData("Validity Check not met!")
                 isShortTermBullish = (executeOption == 11 and isValidityCheckMet)
@@ -372,10 +373,11 @@ class StockScreener:
                         isConfluence = screener.validateConfluence(
                             stock,
                             processedData,
+                            fullData,
                             screeningDictionary,
                             saveDictionary,
                             percentage=insideBarToLookback,
-                            confFilter=(maLength if maLength > 0 else 3) # 1 = Conf up, 2 = Conf Down, 3 = all
+                            confFilter=(maLength if maLength > 0 else 3) # 1 = Conf up, 2 = Conf Down, 3 = all, 4 super confluence (10>20>55 EMA > 200SMA)
                         )
                         if not isConfluence:
                             return returnLegibleData(f"isConfluence:{isConfluence}")
@@ -457,7 +459,8 @@ class StockScreener:
                                 onlyMF=(executeOption == 21 and reversalOption in [5,6]),
                                 hostData=data,
                                 exchangeName=exchangeName,
-                                refreshMFAndFV=(menuOption in ["X"])
+                                refreshMFAndFV=(menuOption in ["X", "C"]),
+                                downloadOnly=True
                             )
                             hostRef.objectDictionaryPrimary[stock] = data.to_dict("split")
                 except np.RankWarning as e: # pragma: no cover 
@@ -563,7 +566,7 @@ class StockScreener:
                         or (executeOption == 9 and hasMinVolumeRatio)
                         or (executeOption == 10 and isPriceRisingByAtLeast2Percent)
                         or (executeOption == 11 and isShortTermBullish)
-                        or (executeOption in [12,13,14,15,16,17,18,19,20,23,24,25,27,28,30,31,32] and isValidityCheckMet)
+                        or (executeOption in [12,13,14,15,16,17,18,19,20,23,24,25,27,28,30,31,32,33,34,35,36,37,38] and isValidityCheckMet)
                         or (executeOption == 21 and (mfiStake > 0 and reversalOption in [3,5]))
                         or (executeOption == 21 and (mfiStake < 0 and reversalOption in [6,7]))
                         or (executeOption == 21 and (fairValueDiff > 0 and reversalOption in [8]))
@@ -621,7 +624,8 @@ class StockScreener:
                                 stock,
                                 onlyMF=(executeOption == 21 and reversalOption in [5,6]),
                                 hostData=data,
-                                exchangeName=exchangeName
+                                exchangeName=exchangeName,
+                                downloadOnly=downloadOnly
                             )
                             hostRef.objectDictionaryPrimary[stock] = data.to_dict("split")
 
@@ -638,28 +642,28 @@ class StockScreener:
             # Capturing Ctr+C Here isn't a great idea
             pass
         except StockDataEmptyException as e: # pragma: no cover
-            if data is None or (data is not None and not data.isnull().values.all(axis=0)[0]):
-                hostRef.default_logger.debug(f"StockDataEmptyException:{stock}: {e}", exc_info=True)
+            # if data is None or (data is not None and not data.isnull().values.all(axis=0)[0]):
+            #     hostRef.default_logger.debug(f"StockDataEmptyException:{stock}: {e}", exc_info=True)
             pass
         except ScreeningStatistics.EligibilityConditionNotMet as e:
-            if userArgsLog:
-                hostRef.default_logger.debug(f"EligibilityConditionNotMet:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"EligibilityConditionNotMet:{stock}: {e}", exc_info=True)
             pass
         except ScreeningStatistics.NotNewlyListed as e: # pragma: no cover
-            if userArgsLog:
-                hostRef.default_logger.debug(f"NotNewlyListed:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"NotNewlyListed:{stock}: {e}", exc_info=True)
             pass
         except ScreeningStatistics.NotAStageTwoStock as e: # pragma: no cover
-            if userArgsLog:
-                hostRef.default_logger.debug(f"NotAStageTwoStock:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"NotAStageTwoStock:{stock}: {e}", exc_info=True)
             pass
         except ScreeningStatistics.NotEnoughVolumeAsPerConfig as e: # pragma: no cover 
-            if userArgsLog:
-                hostRef.default_logger.debug(f"NotEnoughVolumeAsPerConfig:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"NotEnoughVolumeAsPerConfig:{stock}: {e}", exc_info=True)
             pass
         except ScreeningStatistics.DownloadDataOnly as e: # pragma: no cover
-            if userArgsLog:
-                hostRef.default_logger.debug(f"DownloadDataOnly:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"DownloadDataOnly:{stock}: {e}", exc_info=True)
             try:
                 data = hostRef.objectDictionaryPrimary.get(stock)
                 if data is not None:
@@ -667,30 +671,30 @@ class StockScreener:
                     screener.getMutualFundStatus(stock, hostData=data, force=True, exchangeName=exchangeName)
                     hostRef.objectDictionaryPrimary[stock] = data.to_dict("split")
             except Exception as ex:
-                hostRef.default_logger.debug(f"MFIStatus: {stock}:\n{ex}", exc_info=True)
+                # hostRef.default_logger.debug(f"MFIStatus: {stock}:\n{ex}", exc_info=True)
                 pass
             try:
                 screener.getFairValue(stock,hostData=data, force=True,exchangeName=exchangeName)
                 hostRef.objectDictionaryPrimary[stock] = data.to_dict("split")
             except Exception as ex:
-                hostRef.default_logger.debug(f"FairValue: {stock}:\n{ex}", exc_info=True)
+                # hostRef.default_logger.debug(f"FairValue: {stock}:\n{ex}", exc_info=True)
                 pass
             pass
         except ScreeningStatistics.LTPNotInConfiguredRange as e: # pragma: no cover
-            if userArgsLog:
-                hostRef.default_logger.debug(f"LTPNotInConfiguredRange:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"LTPNotInConfiguredRange:{stock}: {e}", exc_info=True)
             pass
         except KeyError as e: # pragma: no cover
-            if userArgsLog:
-                hostRef.default_logger.debug(f"KeyError:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"KeyError:{stock}: {e}", exc_info=True)
             pass
         except OSError as e: # pragma: no cover
-            if userArgsLog:
-                hostRef.default_logger.debug(f"OSError:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"OSError:{stock}: {e}", exc_info=True)
             pass
         except Exception as e:  # pragma: no cover
-            if userArgsLog:
-                hostRef.default_logger.debug(f"Exception:{stock}: {e}", exc_info=True)
+            # if userArgsLog:
+            #     hostRef.default_logger.debug(f"Exception:{stock}: {e}", exc_info=True)
             if testbuild or printCounter:
                 # import traceback
                 # traceback.print_exc()
@@ -705,9 +709,9 @@ class StockScreener:
                 )
         return None
 
-    def performValidityCheckForExecuteOptions(self,executeOption,screener,fullData,screeningDictionary,saveDictionary,processedData,configManager,buySellAll=3):
+    def performValidityCheckForExecuteOptions(self,executeOption,screener,fullData,screeningDictionary,saveDictionary,processedData,configManager,buySellAll=3,intraday_data=None):
         isValid = True
-        if executeOption not in [11,12,13,14,15,16,17,18,19,20,23,24,25,27,28,30,31,32]:
+        if executeOption not in [11,12,13,14,15,16,17,18,19,20,23,24,25,27,28,30,31,32,33,34,35,36,37,38]:
             return True
         if executeOption == 11:
             isValid = screener.validateShortTermBullish(
@@ -751,6 +755,21 @@ class StockScreener:
             isValid = screener.findATRTrailingStops(fullData,sensitivity=configManager.atrTrailingStopSensitivity, atr_period=configManager.atrTrailingStopPeriod,ema_period=configManager.atrTrailingStopEMAPeriod,buySellAll=buySellAll,saveDict=saveDictionary,screenDict=screeningDictionary)
         elif executeOption == 31: # findBuySellSignalsFromATRTrailing # findATRTrailingStops
             isValid = screener.findHighMomentum(processedData)
+        elif executeOption == 32: # findIntradayOpenSetup
+            isValid = screener.findIntradayOpenSetup(processedData,intraday_data,saveDictionary,screeningDictionary,buySellAll=buySellAll)
+        elif executeOption == 33: # findPotentialProfitableEntries
+            isValid = screener.findPotentialProfitableEntries(processedData,fullData, saveDictionary,screeningDictionary)
+        elif executeOption == 34: # findBullishAVWAP
+            isValid = screener.findBullishAVWAP(fullData,screeningDictionary,saveDictionary)
+        elif executeOption == 35: # findPerfectShortSellsFutures
+            isValid = screener.findPerfectShortSellsFutures(fullData)
+        elif executeOption == 36: # findProbableShortSellsFutures
+            isValid = screener.findProbableShortSellsFutures(fullData)
+        elif executeOption == 37: # findShortSellCandidatesForVolumeSMA
+            isValid = screener.findShortSellCandidatesForVolumeSMA(fullData)
+        elif executeOption == 38: # findIntradayShortSellWithPSARVolumeSMA
+            isValid = screener.findIntradayShortSellWithPSARVolumeSMA(fullData,intraday_data)
+
         return isValid        
                     
     def performBasicVolumeChecks(self, executeOption, volumeRatio, screeningDictionary, saveDictionary, processedData, configManager, screener):
@@ -835,13 +854,14 @@ class StockScreener:
         data = None
         hostDataLength = 0 if hostData is None else (0 if "data" not in hostData.keys() else len(hostData["data"]))
         start = None
+        lastTradingDate = PKDateUtilities.tradingDate().strftime("%Y-%m-%d")
         if (period == '1d' or configManager.duration[-1] == "m"):
             if backtestDuration > 0: # We are backtesting
                 start = PKDateUtilities.nthPastTradingDateStringFromFutureDate(backtestDuration)
                 end = PKDateUtilities.nthPastTradingDateStringFromFutureDate(backtestDuration-1)
             else:
                 # Since this is intraday data, we'd just need to start from the last trading session
-                start = PKDateUtilities.tradingDate().strftime("%Y-%m-%d")
+                start = lastTradingDate
                 end = PKDateUtilities.currentDateTime().strftime("%Y-%m-%d")
         if (
                 not shouldCache
@@ -909,7 +929,7 @@ class StockScreener:
 
         if ((shouldCache and not self.isTradingTime and (hostData is None  or hostDataLength == 0)) or downloadOnly) \
             or (shouldCache and hostData is None):  # and backtestDuration == 0 # save only if we're NOT backtesting
-                if start is None and data is not None:
+                if start is None or start is lastTradingDate and data is not None:
                     objectDictionary[stock] = data.to_dict("split")
                 if downloadOnly:
                     with hostRef.processingResultsCounter.get_lock():
@@ -982,12 +1002,12 @@ class StockScreener:
             "Stock",
             "LTP",
             "%Chng",
-            "52Wk H",
-            "52Wk L",
+            "52Wk-H",
+            "52Wk-L",
             "RSI",
             "RSIi",
             "Volume",
-            "22-Pd %",
+            "22-Pd",
             "Consol.",
             "Breakout",
             "MA-Signal",
@@ -1000,12 +1020,12 @@ class StockScreener:
             "Stock": "",
             "LTP": 0,
             "%Chng": 0,
-            "52Wk H": 0,
-            "52Wk L": 0,
+            "52Wk-H": 0,
+            "52Wk-L": 0,
             "RSI": 0,
             "RSIi": 0,
             "Volume": "",
-            "22-Pd %": "",
+            "22-Pd": "",
             "Consol.": "Range:0%",
             "Breakout": "BO: 0 R: 0",
             "MA-Signal": "",
@@ -1018,12 +1038,12 @@ class StockScreener:
             "Stock": "",
             "LTP": 0,
             "%Chng": 0,
-            "52Wk H": 0,
-            "52Wk L": 0,
+            "52Wk-H": 0,
+            "52Wk-L": 0,
             "RSI": 0,
             "RSIi": 0,
             "Volume": "",
-            "22-Pd %": "",
+            "22-Pd": "",
             "Consol.": "Range:0%",
             "Breakout": "BO: 0 R: 0",
             "MA-Signal": "",
