@@ -61,6 +61,7 @@ from PKDevTools.classes.PKDateUtilities import PKDateUtilities
 from PKDevTools.classes.Committer import Committer
 from PKDevTools.classes.SuppressOutput import SuppressOutput
 from PKDevTools.classes.FunctionTimeouts import exit_after
+from PKDevTools.classes.MarketHours import MarketHours
 from tabulate import tabulate
 
 import pkscreener.classes.ConfigManager as ConfigManager
@@ -77,6 +78,9 @@ from PKDevTools.classes.Utils import random_user_agent
 from pkscreener.classes.ArtTexts import getArtText
 from PKDevTools.classes.NSEMarketStatus import NSEMarketStatus
 
+import PIL.Image
+PIL.Image.MAX_IMAGE_PIXELS = None
+
 configManager = ConfigManager.tools()
 configManager.getConfig(ConfigManager.parser)
 nseFetcher = nseStockDataFetcher()
@@ -210,16 +214,17 @@ class tools:
             else:
                 needsWriting = True
             if df is not None and len(df) > 0:
-                df.sort_values(by=["Stock"], ascending=True, inplace=True)
-                df.to_pickle(lastScreened)
-                if choices is not None and df_save is not None:
-                    df_s = df_save.copy()
-                    df_s.reset_index(inplace=True)
-                    newStocks = df_s["Stock"].to_json(orient='records', lines=True).replace("\n","").replace("\"","").split(",")
-                    items.extend(newStocks)
-                    stockList = sorted(list(filter(None,list(set(items)))))
-                    finalStocks = ",".join(stockList)
-                    needsWriting = True
+                with pd.option_context('mode.chained_assignment', None):
+                    df.sort_values(by=["Stock"], ascending=True, inplace=True)
+                    df.to_pickle(lastScreened)
+                    if choices is not None and df_save is not None:
+                        df_s = df_save.copy()
+                        df_s.reset_index(inplace=True)
+                        newStocks = df_s["Stock"].to_json(orient='records', lines=True).replace("\n","").replace("\"","").split(",")
+                        items.extend(newStocks)
+                        stockList = sorted(list(filter(None,list(set(items)))))
+                        finalStocks = ",".join(stockList)
+                        needsWriting = True
             if needsWriting:
                 with open(fileName, 'w') as f:
                     f.write(finalStocks)
@@ -313,6 +318,17 @@ class tools:
             return colorText.BOLD + colorText.GREEN + str(ratio) + "x" + colorText.END
         return colorText.BOLD + colorText.FAIL + (f"{ratio}x" if pd.notna(ratio) else "") + colorText.END
 
+    def getsize_multiline(font,srcText,x=0,y=0):
+        zeroSizeImage = Image.new('RGB',(0, 0), (0,0,0))
+        zeroDraw = ImageDraw.Draw(zeroSizeImage)
+        # zeroDraw = ImageDraw.Draw(zeroSizeImage)
+        left, top, bottom, right = zeroDraw.multiline_textbbox((x,y),srcText,font)
+        return right - left, bottom - top
+
+    def getsize(font,srcText,x=0,y=0):
+        left, top, bottom, right = font.getbbox(srcText)
+        return right - left, bottom - top
+    
     def addQuickWatermark(sourceImage:Image, xVertical=None, dataSrc="", dataSrcFontSize=10):
         width, height = sourceImage.size
         watermarkText = f"© {datetime.date.today().year} pkjmesra | PKScreener"
@@ -544,7 +560,10 @@ class tools:
         for df in dfs_to_print:
             counter += 1
             colPixelRunValue = startColValue
-            if df is None or len(df) == 0 or df.empty:
+            try:
+                if df is None or len(df) == 0:
+                    continue
+            except:
                 continue
             # selected menu options and As of DateTime
             draw.text(
@@ -684,9 +703,9 @@ class tools:
             # Let's go to the next line
             rowPixelRunValue += artfont_line_height + 1
 
-        im = im.resize(im.size, Image.ANTIALIAS, reducing_gap=2)
-        im = tools.addQuickWatermark(im,xVertical,dataSrc="Yahoo; Morningstar, Inc; National Stock Exchange of India Ltd;",dataSrcFontSize=ART_FONT_SIZE)
-        im.save(filename, format="png", bitmap_format="png", optimize=True, quality=20)
+        im = im.resize((int(im.size[0]*configManager.telegramImageCompressionRatio),int(im.size[1]*configManager.telegramImageCompressionRatio)), Image.ANTIALIAS, reducing_gap=2)
+        im = tools.addQuickWatermark(im,xVertical,dataSrc="Yahoo!finance; Morningstar, Inc; National Stock Exchange of India Ltd;TradingHours.com;",dataSrcFontSize=ART_FONT_SIZE)
+        im.save(filename, format=configManager.telegramImageFormat, bitmap_format=configManager.telegramImageFormat, optimize=True, quality=int(configManager.telegramImageQualityPercentage))
         # if 'RUNNER' not in os.environ.keys() and 'PKDevTools_Default_Log_Level' in os.environ.keys():
         # im.show()
 
@@ -766,17 +785,17 @@ class tools:
 
     def afterMarketStockDataExists(intraday=False, forceLoad=False):
         curr = PKDateUtilities.currentDateTime()
-        openTime = curr.replace(hour=9, minute=15)
+        openTime = curr.replace(hour=MarketHours().openHour, minute=MarketHours().openMinute)
         cache_date = PKDateUtilities.previousTradingDate(PKDateUtilities.nextTradingDate(curr)) #curr  # for monday to friday
         weekday = curr.weekday()
         isTrading = PKDateUtilities.isTradingTime()
         if (forceLoad and isTrading) or isTrading:
             #curr = PKDateUtilities.tradingDate()
             cache_date = PKDateUtilities.previousTradingDate(curr) #curr - datetime.timedelta(1)
-        # for monday to friday before 9:15 or between 9:15am to 3:30pm, we're backtesting
+        # for monday to friday before market open or between market open to market close, we're backtesting
         if curr < openTime:
             cache_date = PKDateUtilities.previousTradingDate(curr) # curr - datetime.timedelta(1)
-        if weekday == 0 and curr < openTime:  # for monday before 9:15
+        if weekday == 0 and curr < openTime:  # for monday before market open
             cache_date = PKDateUtilities.previousTradingDate(curr) #curr - datetime.timedelta(3)
         if weekday == 5 or weekday == 6:  # for saturday and sunday
             cache_date = PKDateUtilities.previousTradingDate(curr) # curr - datetime.timedelta(days=weekday - 4)
@@ -812,7 +831,7 @@ class tools:
                     if "RUNNER" not in os.environ.keys():
                         copyFilePath = os.path.join(Archiver.get_user_outputs_dir(), f"copy_{fileName}")
                         cacheFileSize = os.stat(cache_file).st_size if os.path.exists(cache_file) else 0
-                        if os.path.exists(cache_file) and cacheFileSize >= 1024*1024*50:
+                        if os.path.exists(cache_file) and cacheFileSize >= 1024*1024*40:
                             shutil.copy(cache_file,copyFilePath) # copy is the saved source of truth
             except pickle.PicklingError as e:  # pragma: no cover
                 default_logger().debug(e, exc_info=True)
@@ -1154,7 +1173,7 @@ class tools:
         return stockDict,stockDataLoaded
 
     # Save screened results to excel
-    def promptSaveResults(sheetName,df, defaultAnswer=None):
+    def promptSaveResults(sheetName,df, defaultAnswer=None,pastDate=None):
         """
         Tries to save the dataframe output into an excel file.
 
@@ -1179,8 +1198,10 @@ class tools:
             default_logger().debug(e, exc_info=True)
             response = "Y"
         if response is not None and response.upper() != "N":
+            pastDateString = f"{pastDate}_to_" if pastDate is not None else ""
             filename = (
-                "PKScreener-result_"
+                f"PKS_{sheetName}_"
+                + pastDateString
                 + PKDateUtilities.currentDateTime().strftime("%d-%m-%y_%H.%M.%S")
                 + ".xlsx"
             )
@@ -1352,11 +1373,11 @@ class tools:
         m3 = menus()
         m3.renderForMenu(menu,asList=True)
         lMenu =  m3.find(str(respChartPattern))
-        maLength = tools.promptSubMenuOptions(lMenu)
+        maLength = tools.promptSubMenuOptions(lMenu,defaultOption= "4" if respChartPattern == 3 else "1" )
         return maLength
     
     # Prompt for submenu options
-    def promptSubMenuOptions(menu=None):
+    def promptSubMenuOptions(menu=None, defaultOption="1"):
         try:
             tools.promptMenus(menu=menu)
             resp = int(
@@ -1365,7 +1386,7 @@ class tools:
                     + colorText.WARN
                     + """[+] Select Option:"""
                     + colorText.END
-                ) or "1"
+                ) or defaultOption
             )
             if resp >= 0 and resp <= 10:
                 return resp
@@ -1478,9 +1499,9 @@ class tools:
                     input(
                         colorText.BOLD
                         + colorText.WARN
-                        + "\n[+] Enter Percentage within which all MA/EMAs should be (Ideal: 1-2%)? (Default=2): "
+                        + "\n[+] Enter Percentage within which all MA/EMAs should be (Ideal: 0.1-2%)? (Default=0.8): "
                         + colorText.END
-                    ) or "2"
+                    ) or "0.8"
                 )
                 return (resp, percent / 100.0)
             if resp >= 0 and resp <= 9:
