@@ -63,6 +63,7 @@ OWNER_USER = "Itsonlypk"
 from PKDevTools.classes.Telegram import get_secrets
 from PKDevTools.classes.PKDateUtilities import PKDateUtilities
 from PKDevTools.classes.ColorText import colorText
+from PKDevTools.classes.MarketHours import MarketHours
 from pkscreener.classes.MenuOptions import MenuRenderStyle, menu, menus
 from pkscreener.classes.WorkflowManager import run_workflow
 from pkscreener.globals import showSendConfigInfo, showSendHelpInfo
@@ -125,32 +126,34 @@ m2 = menus()
 m3 = menus()
 m4 = menus()
 int_timer = None
+_updater = None
 
 TOP_LEVEL_SCANNER_MENUS = ["X", "B", "MI","DV", "P"]
 TOP_LEVEL_SCANNER_SKIP_MENUS = ["M", "S", "G", "C", "T", "D", "I", "E", "U", "L", "Z", "P"]
 INDEX_SKIP_MENUS = ["W","E","M","Z","0","2","3","4","6","7","9","10","13"]
-SCANNER_SKIP_MENUS_1_TO_6 = ["0","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","42","M","Z"]
-SCANNER_SKIP_MENUS_7_TO_12 = ["0","1","2","3","4","5","6","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","42","M","Z"]
-SCANNER_SKIP_MENUS_13_TO_18 = ["0","1","2","3","4","5","6","7","8","9","10","11","12","19","20","21","22","23","24","25","26","27","28","29","30","31","32","42","M","Z"]
-SCANNER_SKIP_MENUS_19_TO_25 = ["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","22","26","27","28","29","30","31","32","42","M","Z"]
-SCANNER_SKIP_MENUS_26_TO_31 = ["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","32","42","M","Z"]
-SCANNER_MENUS_WITH_NO_SUBMENUS = ["1","2","3","10","11","12","13","14","15","16","17","18","19","20","21","23","24","25","26","27","28","29","30","31","32"]
+SCANNER_SKIP_MENUS_1_TO_6 = ["0","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","M","Z"]
+SCANNER_SKIP_MENUS_7_TO_12 = ["0","1","2","3","4","5","6","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","M","Z"]
+SCANNER_SKIP_MENUS_13_TO_18 = ["0","1","2","3","4","5","6","7","8","9","10","11","12","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","M","Z"]
+SCANNER_SKIP_MENUS_19_TO_25 = ["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","22","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41","42","M","Z"]
+SCANNER_SKIP_MENUS_26_TO_31 = ["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","32","33","34","35","36","37","38","39","40","41","42","M","Z"]
+SCANNER_SKIP_MENUS_32_TO_38 = ["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","39","40","41","42","M","Z"]
+SCANNER_MENUS_WITH_NO_SUBMENUS = ["1","2","3","10","11","12","13","14","15","16","17","18","19","20","21","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41"]
 SCANNER_MENUS_WITH_SUBMENU_SUPPORT = ["6", "7", "21"]
 SCANNER_SUBMENUS_CHILDLEVEL_SUPPORT = {"6":[ "7","10"], "7":[ "3","6","9"]}
 
-INDEX_COMMANDS_SKIP_MENUS_SCANNER = ["W", "E", "M", "Z"]
-INDEX_COMMANDS_SKIP_MENUS_BACKTEST = ["W", "E", "M", "Z", "N", "0", "15"]
+INDEX_COMMANDS_SKIP_MENUS_SCANNER = ["W", "E", "M", "Z", "S"]
+INDEX_COMMANDS_SKIP_MENUS_BACKTEST = ["W", "E", "M", "Z", "S", "N", "0", "15"]
 PIPED_SCAN_SKIP_COMMAND_MENUS =["2", "3", "M", "0"]
 UNSUPPORTED_COMMAND_MENUS =["22","42","M","Z","0"]
-SUPPORTED_COMMAND_MENUS = ["1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32"]
+SUPPORTED_COMMAND_MENUS = ["1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","31","32","33","34","35","36","37","38","39","40","41"]
 
 def initializeIntradayTimer():
     try:
         if (not PKDateUtilities.isTodayHoliday()[0]):
             now = PKDateUtilities.currentDateTime()
-            marketStartTime = PKDateUtilities.currentDateTime(simulate=True,hour=9,minute=14)
-            morning745am = PKDateUtilities.currentDateTime(simulate=True,hour=7,minute=45)
-            if now < marketStartTime and now >= morning745am: # Telegram bot might keep running beyond an hour. So let's start watching around 7:45AM
+            marketStartTime = PKDateUtilities.currentDateTime(simulate=True,hour=MarketHours().openHour,minute=MarketHours().openMinute-1)
+            marketOpenAnHourandHalfPrior = PKDateUtilities.currentDateTime(simulate=True,hour=MarketHours().openHour-2,minute=MarketHours().openMinute+30)
+            if now < marketStartTime and now >= marketOpenAnHourandHalfPrior: # Telegram bot might keep running beyond an hour. So let's start watching around 7:45AM
                 difference = (marketStartTime - now).total_seconds() + 1
                 global int_timer
                 int_timer = threading.Timer(difference, launchIntradayMonitor, args=[])
@@ -293,7 +296,7 @@ def launchIntradayMonitor():
             monitor_proc = Popen(appArgs)
             logger.info(f"{launcher} -a Y -m 'X' -p --telegram launched")
         else:
-            result_outputs = "Monitor is running, but the results are being prepared. Try again in next few seconds."
+            result_outputs = "Intraday Monitor is already running/launching, but the results are being prepared. Try again in the next few seconds."
             logger.info(f"{launcher} -a Y -m 'X' -p --telegram already running")
     except Exception as e:
         result_outputs = "Hmm...It looks like you caught us taking a break! Try again later :-)"
@@ -439,7 +442,7 @@ def Level2(update: Update, context: CallbackContext) -> str:
                 skip=SCANNER_SKIP_MENUS_1_TO_6,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            menuText = menuText + "\n\nN > More options"
+            menuText = menuText + "\n\nP1 > More options"
             menuText = menuText + "\nH > Home"
             mns = m2.renderForMenu(
                 m1.find(selection[1]),
@@ -447,18 +450,18 @@ def Level2(update: Update, context: CallbackContext) -> str:
                 asList=True,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            mns.append(menu().create("N", "More Options", 2))
+            mns.append(menu().create("P1", "More Options", 2))
             mns.append(menu().create("H", "Home", 2))
-        elif selection[1] == "N":
+        elif selection[1] == "P1":
             selection.extend(["", ""])
     elif len(selection) == 3:
-        if selection[2] == "N":
+        if selection[2] == "P1":
             menuText = m2.renderForMenu(
                 m1.find(selection[1]),
                 skip=SCANNER_SKIP_MENUS_7_TO_12,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            menuText = menuText + "\n\nM > More Options"
+            menuText = menuText + "\n\nP2 > More Options"
             menuText = menuText + "\nH > Home"
             mns = m2.renderForMenu(
                 m1.find(selection[1]),
@@ -466,15 +469,15 @@ def Level2(update: Update, context: CallbackContext) -> str:
                 asList=True,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            mns.append(menu().create("M", "More Options", 2))
+            mns.append(menu().create("P2", "More Options", 2))
             mns.append(menu().create("H", "Home", 2))
-        elif selection[2] == "M":
+        elif selection[2] == "P2":
             menuText = m2.renderForMenu(
                 m1.find(selection[1]),
                 skip=SCANNER_SKIP_MENUS_13_TO_18,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            menuText = menuText + "\n\n>> More Options"
+            menuText = menuText + "\n\nP3 > More Options"
             menuText = menuText + "\nH > Home"
             mns = m2.renderForMenu(
                 m1.find(selection[1]),
@@ -482,15 +485,15 @@ def Level2(update: Update, context: CallbackContext) -> str:
                 asList=True,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            mns.append(menu().create(">>", "More Options", 2))
+            mns.append(menu().create("P3", "More Options", 2))
             mns.append(menu().create("H", "Home", 2))
-        elif selection[2] == ">>":
+        elif selection[2] == "P3":
             menuText = m2.renderForMenu(
                 m1.find(selection[1]),
                 skip=SCANNER_SKIP_MENUS_19_TO_25,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            menuText = menuText + "\n\nR > More Options"
+            menuText = menuText + "\n\nP4 > More Options"
             menuText = menuText + "\nH > Home"
             mns = m2.renderForMenu(
                 m1.find(selection[1]),
@@ -498,21 +501,38 @@ def Level2(update: Update, context: CallbackContext) -> str:
                 asList=True,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            mns.append(menu().create("R", "More Options", 2))
+            mns.append(menu().create("P4", "More Options", 2))
             mns.append(menu().create("H", "Home", 2))
-        elif selection[2] == "R":
+        elif selection[2] == "P4":
             menuText = m2.renderForMenu(
                 m1.find(selection[1]),
                 skip=SCANNER_SKIP_MENUS_26_TO_31,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
-            menuText = menuText + "\n\nH > Home"
+            menuText = menuText + "\n\nP5 > More Options"
+            menuText = menuText + "\nH > Home"
             mns = m2.renderForMenu(
                 m1.find(selection[1]),
                 skip=SCANNER_SKIP_MENUS_26_TO_31,
                 asList=True,
                 renderStyle=MenuRenderStyle.STANDALONE,
             )
+            mns.append(menu().create("P5", "More Options", 2))
+            mns.append(menu().create("H", "Home", 2))
+        elif selection[2] == "P5":
+            menuText = m2.renderForMenu(
+                m1.find(selection[1]),
+                skip=SCANNER_SKIP_MENUS_32_TO_38,
+                renderStyle=MenuRenderStyle.STANDALONE,
+            )
+            menuText = menuText + "\n"
+            menuText = menuText + "\nH > Home"
+            mns = m2.renderForMenu(
+                m1.find(selection[1]),
+                skip=SCANNER_SKIP_MENUS_32_TO_38,
+                asList=True,
+                renderStyle=MenuRenderStyle.STANDALONE,
+            )
             mns.append(menu().create("H", "Home", 2))
         elif str(selection[2]).isnumeric():
             preSelection = f"{selection[0]}_{selection[1]}_{selection[2]}"
@@ -735,7 +755,9 @@ def error_handler(update: object, context: CallbackContext) -> None:
     ):  # A newer 2nd instance was registered. We should politely shutdown.
         if (
             timeSinceStarted.total_seconds() >= MINUTES_2_IN_SECONDS
-        ):  # shutdown only if we have been running for over 5 minutes
+        ):  # shutdown only if we have been running for over 2 minutes.
+            # This also prevents this newer instance to get shutdown.
+            # Instead the older instance will shutdown
             print(
                 f"Stopping due to conflict after running for {timeSinceStarted.total_seconds()/60} minutes."
             )
@@ -745,12 +767,18 @@ def error_handler(update: object, context: CallbackContext) -> None:
                     int_timer.cancel()
             except:
                 pass
+                #https://github.com/python-telegram-bot/python-telegram-bot/issues/209
+                # if _updater is not None:
+                #     _updater.stop() # Calling stop from within a handler will cause deadlock
             try:
-                context.application.stop()
-                sys.exit(0)
+                # context.dispatcher.stop()
+                thread.interrupt_main() # causes ctrl + c
+                # sys.exit(0)
             except RuntimeError:
-                context.application.shutdown()
-            sys.exit(0)
+                pass
+            except SystemExit:
+                thread.interrupt_main()
+            # sys.exit(0)
         else:
             print("Other instance running!")
             # context.application.run_polling(allowed_updates=Update.ALL_TYPES)
@@ -1291,13 +1319,14 @@ def addCommandsForMenuItems(application):
 def runpkscreenerbot(availability=True) -> None:
     """Run the bot."""
     # Create the Application and pass it your bot's token.
-    global chat_idADMIN, Channel_Id, bot_available
+    global chat_idADMIN, Channel_Id, bot_available, _updater
     bot_available = availability
     Channel_Id, TOKEN, chat_idADMIN, GITHUB_TOKEN = get_secrets()
     # TOKEN = '1234567'
     # Channel_Id = 1001785195297
     # application = Application.builder().token(TOKEN).build()
     application = Updater(TOKEN)
+    _updater = application
     # Get the dispatcher to register handlers
     dispatcher = application.dispatcher
     # Setup conversation handler with the states FIRST and SECOND
