@@ -23,8 +23,8 @@
     SOFTWARE.
 
 """
-# Pyinstaller compile Windows: pyinstaller --onefile --icon=screenshots\icon.ico pkscreener\pkscreenercli.py  --hidden-import cmath --hidden-import talib.stream --hidden-import numpy --hidden-import pandas --hidden-import alive-progress
-# Pyinstaller compile Linux  : pyinstaller --onefile --icon=screenshots/icon.ico pkscreener/pkscreenercli.py  --hidden-import cmath --hidden-import talib.stream --hidden-import numpy --hidden-import pandas --hidden-import alive-progress
+# Pyinstaller compile Windows: pyinstaller --onefile --icon=screenshots\icon.ico pkscreener\pkscreenercli.py  --hidden-import cmath --hidden-import talib.stream --hidden-import numpy --hidden-import pandas --hidden-import alive_progress
+# Pyinstaller compile Linux  : pyinstaller --onefile --icon=screenshots/icon.ico pkscreener/pkscreenercli.py  --hidden-import cmath --hidden-import talib.stream --hidden-import numpy --hidden-import pandas --hidden-import alive_progress
 import warnings
 warnings.simplefilter("ignore", UserWarning,append=True)
 import argparse
@@ -70,6 +70,7 @@ original__stdout=None
 cron_runs=0
 
 def decorator(func):
+    # @infer_global(new_func)
     def new_func(*args, **kwargs):
         if printenabled:
             try:
@@ -205,6 +206,11 @@ if __name__ == "__main__":
         help="Run in production-build mode",
         required=False,
     )
+    argParser.add_argument(
+        "--progressstatus",
+        help="Pass default progress status that you'd like to get displayed when running the scans",
+        required=False,
+    )
     argParser.add_argument(
         "--runintradayanalysis",
         action="store_true",
@@ -242,6 +248,11 @@ if __name__ == "__main__":
         help="Run with an assumption that this instance is launched via telegram bot",
         required=False,
     )
+    argParser.add_argument(
+        "--triggertimestamp",
+        help="Optionally, send the timestamp value when this was triggered",
+        required=False,
+    )
     argParser.add_argument(
         "-u",
         "--user",
@@ -266,7 +277,14 @@ if __name__ == "__main__":
         help="Piped Menus",
         required=False,
     )
-    argsv = argParser.parse_known_args()
+
+    def get_debug_args():
+        return None
+        # return " -a Y -e -l -o X:12:30:D:D:D:D:D".split(" ")
+
+    args = get_debug_args()
+    argsv = argParser.parse_known_args(args=args)
+    # argsv = argParser.parse_known_args()
     args = argsv[0]
     # if sys.argv[0].endswith(".py"):
     #     args.monitor = 'X'
@@ -279,10 +297,17 @@ if __name__ == "__main__":
     elapsed_time = None
     configManager = ConfigManager.tools()
 
+
 def exitGracefully():
     from PKDevTools.classes import Archiver
     from pkscreener.globals import resetConfigToDefault
-    filePath = os.path.join(Archiver.get_user_outputs_dir(), "monitor_outputs")
+    filePath = None
+    try:
+        filePath = os.path.join(Archiver.get_user_outputs_dir(), "monitor_outputs")
+    except:
+        pass
+    if filePath is None:
+        return
     index = 0
     while index < configManager.maxDashboardWidgetsPerRow*configManager.maxNumResultRowsInMonitor:
         try:
@@ -366,7 +391,7 @@ def warnAboutDependencies():
             input("Press any key to try anyway...")
     
 def runApplication():
-    from pkscreener.globals import main, sendQuickScanResult,sendMessageToTelegramChannel, sendGlobalMarketBarometer, updateMenuChoiceHierarchy, isInterrupted, refreshStockData, closeWorkersAndExit
+    from pkscreener.globals import main, sendQuickScanResult,sendMessageToTelegramChannel, sendGlobalMarketBarometer, updateMenuChoiceHierarchy, isInterrupted, refreshStockData, closeWorkersAndExit, resetUserMenuChoiceOptions
     # From a previous call to main with args, it may have been mutated.
     # Let's stock to the original args passed by user
     try:
@@ -374,8 +399,21 @@ def runApplication():
         savedPipedArgs = args.pipedmenus if args is not None and args.pipedmenus is not None else None
     except:
         pass
-    argsv = argParser.parse_known_args()
+    global results, resultStocks, plainResults, dbTimestamp, elapsed_time, start_time
+    from pkscreener.classes.MenuOptions import menus, PREDEFINED_PIPED_MENU_OPTIONS,PREDEFINED_SCAN_MENU_VALUES
+    args = get_debug_args()
+    argsv = argParser.parse_known_args(args=args)
+    # argsv = argParser.parse_known_args()
     args = argsv[0]
+    if args.user is None:
+        from PKDevTools.classes.Telegram import get_secrets
+        Channel_Id, _, _, _ = get_secrets()
+        if Channel_Id is not None:
+            args.user = int(f"-{Channel_Id}")
+    if args.triggertimestamp is None:
+        args.triggertimestamp = int(PKDateUtilities.currentDateTimestamp())
+    else:
+        args.triggertimestamp = int(args.triggertimestamp)
     if args.systemlaunched:
         args.systemlaunched = args.options
     
@@ -384,20 +422,69 @@ def runApplication():
     #     args.answerdefault = 'Y'
     args.pipedmenus = savedPipedArgs
     if args.options is not None:
-        args.options = args.options.replace("::",":")
+        args.options = args.options.replace("::",":").replace("\"","").replace("'","")
+        if args.options.upper().startswith("C") or "C:" in args.options.upper():
+            args.runintradayanalysis = True
+        try:
+            if args.systemlaunched:
+                choices = f"--systemlaunched -a y -e -o '{args.options.replace('C:','X:').replace('D:','')}'"
+                indexNum = PREDEFINED_SCAN_MENU_VALUES.index(choices)
+                choices = f"{'P_1_'+str(indexNum +1) if '>|' in choices else choices}"
+                args.progressstatus = f"[+] {choices} => Running {choices}"
+        except:
+            choices = ""
+            pass
+        
     if args.runintradayanalysis:
-        from pkscreener.classes.MenuOptions import menus
-        runOptions = menus.allMenus(topLevel="C", index=12)
-        optionalFinalOutcome_df = None
+        maxdisplayresults = configManager.maxdisplayresults
+        configManager.maxdisplayresults = 2000
+        configManager.setConfig(ConfigManager.parser, default=True, showFileCreatedText=False)
+        runOptions = []
+        otherMenus = []
+        if len(args.options.split(":")) >= 4:
+            runOptions = [args.options]
+        else:
+            runOptions = PREDEFINED_PIPED_MENU_OPTIONS
+            otherMenus =  menus.allMenus(topLevel="C", index=12)
+            if len(otherMenus) > 0:
+                runOptions.extend(otherMenus)
+        import pandas as pd
+        optionalFinalOutcome_df = pd.DataFrame()
         import pkscreener.classes.Utility as Utility
         import pandas as pd
         # Delete any existing data from the previous run.
         configManager.deleteFileWithPattern(pattern="stock_data_*.pkl")
+        analysis_index = 1
         for runOption in runOptions:
+            try:
+                runOptionName = f"--systemlaunched -a y -e -o '{runOption.replace('C:','X:').replace('D:','D:')}'"
+                indexNum = PREDEFINED_SCAN_MENU_VALUES.index(runOptionName)
+                runOptionName = f"{'[+] P_1_'+str(indexNum +1) if '>|' in runOption else runOption}"
+            except:
+                runOptionName = ""
+                pass
+            args.progressstatus = f"{runOptionName} => Running Intraday Analysis: {analysis_index} of {len(runOptions)}..."
+            analysisOptions = runOption.split("|")
+            analysisOptions[-1] = analysisOptions[-1].replace("X:","C:")
+            runOption = "|".join(analysisOptions)
             args.options = runOption
             try:
-                optionalFinalOutcome_df,_ = main(userArgs=args,optionalFinalOutcome_df=optionalFinalOutcome_df)
-                if "EoDDiff" not in optionalFinalOutcome_df.columns:
+                results,plainResults = main(userArgs=args,optionalFinalOutcome_df=optionalFinalOutcome_df)
+                if args.pipedmenus is not None:
+                    while args.pipedmenus is not None:
+                        results, plainResults = main(userArgs=args)
+                if isInterrupted():
+                    closeWorkersAndExit()
+                    exitGracefully()
+                    sys.exit(0)
+                runPipedScans = True
+                while runPipedScans:
+                    runPipedScans = pipeResults(plainResults,args)
+                    if runPipedScans:
+                        results, plainResults = main(userArgs=args,optionalFinalOutcome_df=optionalFinalOutcome_df)
+                if results is not None and len(results) >= len(optionalFinalOutcome_df):
+                    optionalFinalOutcome_df = results
+                if optionalFinalOutcome_df is not None and "EoDDiff" not in optionalFinalOutcome_df.columns:
                     # Somehow the file must have been corrupted. Let's re-download
                     configManager.deleteFileWithPattern(pattern="stock_data_*.pkl")
                     configManager.deleteFileWithPattern(pattern="intraday_stock_data_*.pkl")
@@ -407,49 +494,67 @@ def runApplication():
                 OutputControls().printOutput(e)
                 if args.log:
                     traceback.print_exc()
-        if optionalFinalOutcome_df is not None:
+            resetUserMenuChoiceOptions()
+            analysis_index += 1
+
+        configManager.maxdisplayresults = maxdisplayresults
+        configManager.setConfig(ConfigManager.parser, default=True, showFileCreatedText=False)
+        if optionalFinalOutcome_df is not None and not optionalFinalOutcome_df.empty:
             final_df = None
-            optionalFinalOutcome_df.drop('FairValue', axis=1, inplace=True, errors="ignore")
-            df_grouped = optionalFinalOutcome_df.groupby("Stock")
-            for stock, df_group in df_grouped:
-                if stock == "PORTFOLIO":
-                    if final_df is None:
-                        final_df = df_group[["Pattern","LTP","SqrOffLTP","SqrOffDiff","EoDLTP","EoDDiff","DayHigh","DayHighDiff"]]
-                    else:
-                        final_df = pd.concat([final_df, df_group[["Pattern","LTP","SqrOffLTP","SqrOffDiff","EoDLTP","EoDDiff","DayHigh","DayHighDiff"]]], axis=0)
-            final_df.rename(
-                columns={
-                    "LTP": "Morning Portfolio",
-                    "SqrOffLTP": "SqrOff Portfolio",
-                    "EoDLTP": "EoD Portfolio",
-                    },
-                    inplace=True,
-                )
-            mark_down = colorText.miniTabulator().tabulate(
-                                final_df,
-                                headers="keys",
-                                tablefmt=colorText.No_Pad_GridFormat,
-                                showindex = False
-                            ).encode("utf-8").decode(Utility.STD_ENCODING)
-            OutputControls().printOutput(mark_down)
-            sendQuickScanResult(menuChoiceHierarchy="IntradayAnalysis",
-                                user="-1001785195297",
-                                tabulated_results=mark_down,
-                                markdown_results=mark_down,
-                                caption="IntradayAnalysis - Morning alert vs Market Close",
-                                pngName= f"PKS_IA_{PKDateUtilities.currentDateTime().strftime('%Y-%m-%d_%H:%M:%S')}",
-                                pngExtension= ".png"
-                                )
+            try:
+                optionalFinalOutcome_df.drop('FairValue', axis=1, inplace=True, errors="ignore")
+                df_grouped = optionalFinalOutcome_df.groupby("Stock")
+                for stock, df_group in df_grouped:
+                    if stock == "BASKET":
+                        if final_df is None:
+                            final_df = df_group[["Pattern","LTP","LTP@Alert","SqrOffLTP","SqrOffDiff","EoDDiff","DayHigh","DayHighDiff"]]
+                        else:
+                            final_df = pd.concat([final_df, df_group[["Pattern","LTP","LTP@Alert","SqrOffLTP","SqrOffDiff","EoDDiff","DayHigh","DayHighDiff"]]], axis=0)
+            except:
+                pass
+            if final_df is not None and not final_df.empty:
+                with pd.option_context('mode.chained_assignment', None):
+                    final_df = final_df[["Pattern","LTP@Alert","LTP","EoDDiff","SqrOffLTP","SqrOffDiff","DayHigh","DayHighDiff"]]
+                    final_df.rename(
+                        columns={
+                            "Pattern": "Scan Name",
+                            "LTP@Alert": "Basket Value@Alert",
+                            "LTP": "Basket Value@EOD",
+                            "SqrOffLTP": "Basket Value@SqrOff",
+                            "DayHigh": "Basket Value@DayHigh",
+                            },
+                            inplace=True,
+                        )
+                    final_df.dropna(inplace=True)
+                mark_down = colorText.miniTabulator().tabulate(
+                                    final_df,
+                                    headers="keys",
+                                    tablefmt=colorText.No_Pad_GridFormat,
+                                    showindex = False
+                                ).encode("utf-8").decode(Utility.STD_ENCODING)
+                OutputControls().printOutput(mark_down)
+                from PKDevTools.classes.Telegram import get_secrets
+                Channel_Id, _, _, _ = get_secrets()
+                sendQuickScanResult(menuChoiceHierarchy="IntradayAnalysis",
+                                    user=int(f"-{Channel_Id}"),
+                                    tabulated_results=mark_down,
+                                    markdown_results=mark_down,
+                                    caption="IntradayAnalysis - Morning alert vs Market Close",
+                                    pngName= f"PKS_IA_{PKDateUtilities.currentDateTime().strftime('%Y-%m-%d_%H:%M:%S')}",
+                                    pngExtension= ".png",
+                                    forceSend=True
+                                    )
     else:
         if args.barometer:
             sendGlobalMarketBarometer(userArgs=args)
         else:
-            global results, resultStocks, plainResults, dbTimestamp, elapsed_time, start_time
             monitorOption_org = ""
             # args.monitor = configManager.defaultMonitorOptions
             if args.monitor:
+                args.monitor = args.monitor.replace("::",":").replace("\"","").replace("'","")
                 configManager.getConfig(ConfigManager.parser)
                 args.answerdefault = args.answerdefault or 'Y'
+                MarketMonitor().hiddenColumns = configManager.alwaysHiddenDisplayColumns
                 if MarketMonitor().monitorIndex == 0:
                     dbTimestamp = PKDateUtilities.currentDateTime().strftime("%H:%M:%S")
                     elapsed_time = 0
@@ -459,7 +564,7 @@ def runApplication():
                         elapsed_time = round(time.time() - start_time,2)
                         start_time = time.time()
                 monitorOption_org = MarketMonitor().currentMonitorOption()
-                monitorOption = monitorOption_org
+                monitorOption = monitorOption_org.replace("::",":").replace("\"","").replace("'","")
                 monitorOption = checkIntradayComponent(args, monitorOption)
                 if monitorOption.startswith("|"):
                     monitorOption = monitorOption[1:]
@@ -489,8 +594,9 @@ def runApplication():
                         resultStocks = ",".join(resultStocks)
                         monitorOption = f"{monitorOption}:{resultStocks}"
                 args.options = monitorOption.replace("::",":")
-                # (previousCandleDuration != configManager.duration) or 
-                if (MarketMonitor().monitorIndex == 1 and args.options is not None and plainResults is not None):
+                fullMonitorMode = MarketMonitor().monitorIndex == 1 and args.options is not None and plainResults is not None
+                partMonitorMode = len(MarketMonitor().monitors) == 1 and args.options is not None and plainResults is not None
+                if (fullMonitorMode or partMonitorMode):
                     # Load the stock data afresh for each cycle
                     refreshStockData(args.options)
             try:
@@ -526,6 +632,8 @@ def runApplication():
                 sys.exit(0)
             except Exception as e:
                 default_logger().debug(e, exc_info=True)
+                if args.log:
+                    traceback.print_exc()
                 # Probably user cancelled an operation by choosing a cancel sub-menu somewhere
                 pass
             if plainResults is not None and not plainResults.empty:
@@ -563,6 +671,8 @@ def checkIntradayComponent(args, monitorOption):
 
 
 def pipeResults(prevOutput,args):
+    if args is None or args.options is None:
+        return False
     nextOnes = args.options.split(">")
     hasFoundStocks = False
     if len(nextOnes) > 1:
@@ -582,7 +692,7 @@ def pipeResults(prevOutput,args):
         if monitorOption.startswith("|"):
             monitorOption = monitorOption.replace("|","")
             monitorOptions = monitorOption.split(":")
-            if "X" in monitorOptions[0].upper() and monitorOptions[1] != "0":
+            if monitorOptions[0].upper() in ["X","C"] and monitorOptions[1] != "0":
                 monitorOptions[1] = "0"
                 monitorOption = ":".join(monitorOptions)
             if "B" in monitorOptions[0].upper() and monitorOptions[1] != "30":
@@ -606,7 +716,7 @@ def pipeResults(prevOutput,args):
     return False
 
 def pkscreenercli():
-    global originalStdOut
+    global originalStdOut, args
     if sys.platform.startswith("darwin"):
         try:
             multiprocessing.set_start_method("fork")
@@ -619,10 +729,15 @@ def pkscreenercli():
                 traceback.print_exc()
             pass
     try:
-        OutputControls(enableMultipleLineOutput=(args is None or args.monitor is None)).printOutput("",end="\r")
+        OutputControls(enableMultipleLineOutput=(args is None or args.monitor is None or args.runintradayanalysis)).printOutput("",end="\r")
         configManager.getConfig(ConfigManager.parser)
         import atexit
         atexit.register(exitGracefully)
+        # Set the trigger timestamp
+        if args.triggertimestamp is None:
+            args.triggertimestamp = int(PKDateUtilities.currentDateTimestamp())
+        else:
+            args.triggertimestamp = int(args.triggertimestamp)
         # configManager.restartRequestsCache()
         # args.monitor = configManager.defaultMonitorOptions
         if args.monitor is not None:
@@ -630,7 +745,9 @@ def pkscreenercli():
                         maxNumResultsPerRow=configManager.maxDashboardWidgetsPerRow,
                         maxNumColsInEachResult=6,
                         maxNumRowsInEachResult=10,
-                        maxNumResultRowsInMonitor=configManager.maxNumResultRowsInMonitor)
+                        maxNumResultRowsInMonitor=configManager.maxNumResultRowsInMonitor,
+                        pinnedIntervalWaitSeconds=configManager.pinnedMonitorSleepIntervalSeconds,
+                        alertOptions=configManager.soundAlertForMonitorOptions.split("~"))
 
         if args.log or configManager.logsEnabled:
             setupLogger(shouldLog=True, trace=args.testbuild)
@@ -655,7 +772,23 @@ def pkscreenercli():
             Utility.tools.clearScreen(userArgs=args, clearAlways=True)
         warnAboutDependencies()
         if args.prodbuild:
-            disableSysOut()
+            if args.options and len(args.options.split(":")) > 0:
+                indexOption = 0
+                doNotDisableGlobalPrint = False
+                while indexOption <= 15: # Max integer menu index in level1_X_MenuDict in MenuOptions.py
+                    # Menu option 30 is for ATR trailing stops which uses vectorbt
+                    # which in turn uses numba which requires print function to be inferred globally
+                    # If we try to override print with new_func, it expects this new_func
+                    # to be globally available. So to avoid these changes, let's just skip
+                    # prodmode for menu option 30
+                    doNotDisableGlobalPrint = f":{indexOption}:30:" in args.options
+                    if doNotDisableGlobalPrint:
+                        break
+                    indexOption += 1
+                if not doNotDisableGlobalPrint:
+                    disableSysOut()
+            else:
+                disableSysOut()
 
         if not configManager.checkConfigFile():
             configManager.setConfig(
@@ -672,11 +805,11 @@ def pkscreenercli():
                 if os.path.exists(filePath):
                     default_logger().info("monitor_outputs_1.txt already exists! This means an instance may already be running. Exiting now...")
                     # Since the file exists, it means, there is another instance running
-                    sys.exit(0)
+                    return
             else:
                 # It should have been launched only during the trading hours
                 default_logger().info("--telegram option must be launched ONLY during NSE trading hours. Exiting now...")
-                sys.exit(0)
+                return
         # Check and see if we're running only the telegram bot
         if args.bot:
             from pkscreener import pkscreenerbot
@@ -714,6 +847,7 @@ def pkscreenercli():
                 + "[+] Download ONLY mode! Stocks will not be screened!"
                 + colorText.END
             )
+            configManager.restartRequestsCache()
             if args.intraday is None:
                 configManager.toggleConfig(candleDuration="1d", clearCache=False)
             runApplication()
@@ -723,7 +857,13 @@ def pkscreenercli():
             sys.exit(0)
         else:
             runApplicationForScreening()
-    except:
+    except Exception as e:
+        if "RUNNER" not in os.environ.keys() and ('PKDevTools_Default_Log_Level' in os.environ.keys() and os.environ["PKDevTools_Default_Log_Level"] != str(log.logging.NOTSET)):
+                OutputControls().printOutput(
+                    "[+] RuntimeError with 'multiprocessing'.\n[+] Please contact the Developer, if this does not work!"
+                )
+                OutputControls().printOutput(e)
+                traceback.print_exc()
         pass
 
 def runLoopOnScheduleOrStdApplication(hasCronInterval):
